package com.gx.obe.component.text;

import java.util.function.Supplier;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import com.gx.obe.ColorConstants;
import com.gx.obe.bind.prop.Prop;
import com.gx.obe.bind.view.ChangeListener;
import com.gx.obe.bind.view.MView;
import com.gx.obe.component.verify.Verify;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description:
 * @author chenxw
 */
public class DocumentText extends Composite implements MView<String>, Verify<String> {
	
	/**
	 * @Description: 标题
	 */
	private final Prop<String> title = new Prop<>();
	/**
	 * @Description: 内容
	 */
	private final Prop<String> content = new Prop<>();
	/**
	 * @Description: 可编辑
	 */
	private final Prop<Boolean> editable = new Prop<>(false);
	
	/**
	 * @Description: 获取内容文本框
	 */
	private Supplier<Control> contentTextGet;
	
	/**
	 * @author chenxw
	 * @param parent
	 * @param style
	 */
	public DocumentText(Composite parent, int style) {
		super(parent, style);
		
		setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		setBackgroundMode(SWT.INHERIT_FORCE);
		
		GridLayout gl_composite = new GridLayout(1, false);
		gl_composite.marginHeight = 0;
		gl_composite.marginWidth = 50;
		setLayout(gl_composite);
		
		Label titleLabel = new Label(this, SWT.NONE);
		titleLabel.setFont(SWTResourceManager.getFont("微软雅黑", 12, SWT.NORMAL));
		titleLabel.setForeground(ColorConstants.TITLE_FONT_COLOR);
		titleLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		title.bind(titleLabel::setText);
		
		StyledText contentText = new StyledText(this, SWT.WRAP | SWT.MULTI);
		contentText.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		contentText.setLineSpacing(10);
		contentText.setIndent(30);
		contentText.setFont(SWTResourceManager.getFont("微软雅黑", 12, SWT.NORMAL));
		contentText.setForeground(ColorConstants.FONT_BLACK_COLOR);
		editable.bind(contentText::setEditable);
		content.bindBidirectional(contentText::setText, contentText::getText, t -> contentText.addModifyListener(e -> t.change()));
		contentTextGet = () -> contentText;
		
		Label memoTips = new Label(this, SWT.RIGHT);
		memoTips.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		content.bind(t -> memoTips.setText("(" + t.length() + "/ " + "500)"));
	}
	
	/**
	 * @Description: 设置标题
	 * @author chenxw
	 * @param title
	 */
	public void setTitle(String title) {
		this.title.set(title);
	}
	
	/**
	 * @Description: 设置可编辑状态
	 * @author chenxw
	 * @param editable 
	 */
	public void setEditable(boolean editable) {
		this.editable.set(editable);
	}
	
	@Override
	public void addChangeListener(ChangeListener changeListener) {
		content.addChangeListener(changeListener);
	}
	
	@Override
	public String get() {
		return content.get();
	}
	
	@Override
	public void set(String content) {
		this.content.set(content);
	}
	
	@Override
	public Supplier<Control> getVerifyControl() {
		return contentTextGet;
	}
	
	@Override
	protected void checkSubclass() {}
	
}
