package com.gx.obe.component.text;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;

import com.swtdesigner.SWTResourceManager;

public class FStyledText extends StyledText {
	
	private StringBuilder textBuilder = new StringBuilder();
	
	private List<StyleRange> styleRangeList = new ArrayList<>();
	
	public FStyledText(Composite parent) {
		super(parent, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL | SWT.READ_ONLY);
		this.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.NORMAL));
	}
	
	public void clear() {
		textBuilder.delete(0, textBuilder.length());
		styleRangeList.clear();
	}
	
	public void refresh() {
		setText(textBuilder.toString());
		styleRangeList.forEach(this::setStyleRange);
	}
	
	public void add(String title, String content, Color contentForeground, int contentFontStyle) {
		if (textBuilder.length() != 0) textBuilder.append("    ");
		addItem(title, content, contentForeground, contentFontStyle);
		textBuilder.append(title).append(content);
	}
	
	public void addWrap(String title, String content, Color contentForeground, int contentFontStyle) {
		textBuilder.append("\r\n");
		addItem(title, content, contentForeground, contentFontStyle);
		textBuilder.append(title).append(content);
	}
	
	private void addItem(String title, String content, Color contentForeground, int contentFontStyle) {
		StyleRange titleStyleRange = new StyleRange();
		titleStyleRange.start = textBuilder.length();
		titleStyleRange.length = title.length();
		titleStyleRange.foreground = SWTResourceManager.getColor(SWT.COLOR_BLUE);
		titleStyleRange.fontStyle = SWT.BOLD;
		styleRangeList.add(titleStyleRange);
		
		StyleRange contentStyleRange = new StyleRange();
		contentStyleRange.start = textBuilder.length() + title.length();
		contentStyleRange.length = content.length();
		contentStyleRange.foreground = contentForeground;
		contentStyleRange.fontStyle = contentFontStyle;
		styleRangeList.add(contentStyleRange);
	}
	
	@Override
	protected void checkSubclass() {}
	
}
